{{

Global functions used in the RGDISP1 project

}}

VAR
                                                                     
  
CON

  ' File type definitions
  #0
  _UNKNOWN     ' File type unrecognised
  _IMAGE       ' File with .img extension
  _ANIMATION   ' File with .ani extension
  _SYSTEM      ' File with .sys extension
  _FONT        ' File with .fnt extension
  
OBJ

  comms : "SerialMirror"

PUB Wait(millis)
{{
    Wait millis milliseconds
}}

  ' 1 millisecond is 1000 clock ticks
  'waitcnt ((clkfreq / 1000) * millis + cnt)
  waitcnt(((clkfreq / 1_000 * millis - 3932) #> 381) + cnt)


PUB GetTimeStamp | clkptr

  return cnt

{
PUB TimeDiff(p_TimeVar) | RightNow, ClockTicks

  ' Written by StefanL38

  RightNow := cnt

  if RightNow < p_TimeVar                          
    ClockTicks := ||($FFFFFFFF - RightNow + p_TimeVar)  
  else
    ClockTicks := ||(RightNow - p_TimeVar)

  result := ClockTicks / (clkfreq / 1_000) 'calculate milliseconds
}

PUB TimeDiff(p_TimeVar) | RightNow, ClockTicks

  ' Written by StefanL38
  ' Updated so that the value passed is now a pointer to a long,
  ' which is automatically updated to the current value of cnt.
  ' This should simplify usage

  RightNow := cnt

  if RightNow < p_TimeVar                          
    ClockTicks := ||($FFFFFFFF - RightNow + LONG[p_TimeVar])
  else
    ClockTicks := ||(RightNow - LONG[p_TimeVar])

  LONG[p_TimeVar] := RightNow
  
  result := ClockTicks / (clkfreq / 1_000) 'calculate milliseconds

PUB DEC2BCD(number) | temp
{{
    Convert number to its BCD equivalent (8 bits)
}}

  ' Get tens
  temp := 0
  repeat while number => 10
    number -= 10
    temp++
  ' Shift tens (now in temp) up by 4 bits
  temp := temp << 4
  ' Add units (remainder, in number)
  temp := temp + number
  return temp

PUB BCD2DEC(number, mask) | temp1, temp2
{{
    Convert bcd encoded byte to its decimal equivalent (8 bits)
    Use mask to mask specific bits to be extracted eg if only certain bits are valid
    Eg , 2 bits in upper nibble, all bits in lower, mask should be %0011111
}}
  number := number & mask
  ' LOWER NIBBLE FIRST mask off upper 4 bits
  temp1 := number & %00001111
  ' UPPER NIBBLE shift right 4 bits, becomes tens
  temp2 := number >> 4
  ' Calculate and return decimal value
  return (temp2 * 10) + temp1

PUB Num2ASC(n)
{{
    Given a numeric value in range 0-9 this function returns its
    ASCII equivalent as in 0=>48 etc. Returns false if n is not a
    single digit value
}}
  if n =< 0 and n=>9
    return 48+n
  else
    return false ' Return a zero value


PUB Pow(val,pwr)
{{

    UNTESTED
    Raises val to pow e.g., Pow(2,2) = 4

}}

  result := val
  repeat while pwr-- > 1
    result := result * val
  return result
    
PUB Num2String(value,pad,digits,str) | m
{{
    Convert number n to string, 4 digits max
    Actual number of digits specified by 'digits', i.e. if n is 2 and digits
    is 4 then the string returned will be "0002", if n is 400 and digits
    is 2 then the string returned will be "00". Beware!!
    If 'pad' is true, then leading zeros will be added, otherwise they will
    not. Note that this can be affected by the above as well!!
    eg pad = false, digits=1, n=120 => "0"+"\0"!!!
}}
  
  if value < 0
    -value
    byte[str++] := "-"

  m := Pow(10,digits-1)
  
  repeat digits
    if value => m
      byte[str++] := value / m + "0"
      value //= m
      result~~
    elseif result or m == 1
      byte[str++] := "0"
    else
      if pad
        byte[str++] := "0"
    m /= 10

  byte[str] := 0

PUB CopyString(str1,str2)
{{

    Copies string str2 to str1 - str1 original contents erased

}}

  repeat strsize(str2)
    byte[str1++] := byte[str2++]
  ' Terminate str1
  byte[str1] := 0

PUB Concat(str1,str2,str3)
{{

    Concatenates strings str1 and str2, result in str3
    Make sure there's enough space in str3!!

}}
                          
  ' Copy 1 to 3
  repeat strsize(str1)
    byte[str3++] := byte[str1++]

  ' Concat 3 and 2
  repeat strsize(str2)
    byte[str3++] := byte[str2++]

  ' Terminate str3
  byte[str3] := 0

PUB GetDayStr(weekday, str1)

  CASE weekday
    1 : CopyString(str1,string("MON"))
    2 : CopyString(str1,string("TUE"))
    3 : CopyString(str1,string("WED"))
    4 : CopyString(str1,string("THU"))
    5 : CopyString(str1,string("FRI"))
    6 : CopyString(str1,string("SAT"))
    7 : CopyString(str1,string("SUN"))

PUB GetMonthStr(month, str1)

  CASE month
    1  : CopyString(str1,string("JAN "))
    2  : CopyString(str1,string("FEB "))
    3  : CopyString(str1,string("MAR "))
    4  : CopyString(str1,string("APR "))
    5  : CopyString(str1,string("MAY "))
    6  : CopyString(str1,string("JUN "))
    7  : CopyString(str1,string("JUL "))
    8  : CopyString(str1,string("AUG "))
    9  : CopyString(str1,string("SEP "))
    10 : CopyString(str1,string("OCT "))
    11 : CopyString(str1,string("NOV "))
    12 : CopyString(str1,string("DEC ")) 

PUB FileType(str1)

  ' Given the name of a file this function returns its type
  ' Note that although the filesystem only returns UPPER case characters, we
  ' still need to check upper and lower cases here as we don't know how the config
  ' data are stored (actually this was a gaff on my part RH 12.10.08)
  repeat strsize(str1)
    if byte[str1++] == "."
      if strcomp(str1,string("IMG")) or strcomp(str1,string("img"))
        return _IMAGE     
      if strcomp(str1,string("ANI")) or strcomp(str1,string("ani"))
        return _ANIMATION
      if strcomp(str1,string("SYS")) or strcomp(str1,string("sys"))
        return _SYSTEM
      if strcomp(str1,string("FNT")) or strcomp(str1,string("fnt"))
        return _FONT

  return _UNKNOWN ' (=0 =false)

PUB DropExt(str1, str2)

  ' Drops the extension from the filename supplied eg
  ' "engt.ani" becomes "engt"
  ' Modified version returned in str2
  ' Used for some display functions
                                     
  repeat strsize(str1)
    byte[str2++] := byte[str1++]
    if byte[str1] == "." or byte[str1] == 0
      byte[str2]~
      return

PUB ToUpper(str1)

  ' Converts lowercase characters in str1 to uppercase
  ' Modifies str1
  repeat strsize(str1)
    if byte[str1] => 97 and byte[str1] =< 122
      byte[str1] -= 32
      str1++

PUB AverageArray(arrptr,n) : average

  ' Average the values of LONG array pointed to by 'arrptr' containing
  ' 'n' elements

  repeat n
    average += long[arrptr]
    arrptr += 4
    
  average /= n

PUB ShiftArray(arrptr,n)

  ' Shifts all values in an array of size n left by ONE index
  ' e.g., array[0] becomes array[1], array[1] becomes array[2]
  ' and so on. This has been implemented primarily to support
  ' averaging routines such as that used by the voltage display
  ' routine

  repeat n-1
    long[arrptr] := long[arrptr + 4]
    arrptr += 4

PUB C2F(t)

  ' Convert centigrade to farenheit
  return (180 * t) / 100 + 32